@@ -3,7 +3,6 @@
 # Distributed under the terms of "New BSD License", see the LICENSE file.
 
 import os
-import posixpath
 
 import ipywidgets as widgets
 import matplotlib.pyplot as plt
@@ -13,10 +12,10 @@ import numpy as np
 import pandas
 from IPython.core.display import display, HTML
 
-from pyiron_atomistics import Atoms
-from pyiron_atomistics.atomistics.master.murnaghan import Murnaghan
 from pyiron_base import Project as BaseProject
 from pyiron_base.generic.filedata import FileData
+from pyiron_gui.wrapper.widgets import ObjectWidget, NumpyWidget
+from pyiron_gui.wrapper.wrapper import PyironWrapper, BaseWrapper
 
 __author__ = "Niklas Siemer"
 __copyright__ = (
@@ -30,103 +29,22 @@ __status__ = "development"
 __date__ = "Feb 02, 2021"
 
 
-class PyironWrapper:
-
-    """Simple wrapper for pyiron objects which extends for basic pyiron functionality (list_nodes ...)"""
-
-    _with_self_representation = {
-        "structure": Atoms,
-        "murnaghan": Murnaghan
-    }
-
-    def __init__(self, pyi_obj, project, rel_path=""):
-        self._wrapped_object = pyi_obj
-        self._project = project
-        if not hasattr(pyi_obj, 'path'):
-            self._path = self._project.path
-        self._rel_path = rel_path
-        # print("init:" + self.path)
-        self._type = None
-        for name, cls in self._with_self_representation.items():
-            if isinstance(pyi_obj, cls):
-                self._type = name
-
-    @property
-    def has_self_representation(self):
-        return self._type is not None
-
-    @staticmethod
-    def _empty_list():
-        return []
-
-    @property
-    def name(self):
-        return self._type
-
-    @property
-    def project(self):
-        if hasattr(self._wrapped_object, 'project'):
-            return self._wrapped_object.project
-        return self._project
-
-    @property
-    def path(self):
-        if hasattr(self._wrapped_object, 'path'):
-            return self._wrapped_object.path
-        if hasattr(self.project, 'path'):
-            return posixpath.join(self.project.path, self._rel_path)
-        raise AttributeError
-
-    def __getitem__(self, item):
-        try:
-            return self._wrapped_object[item]
-        except (IndexError, KeyError, TypeError):
-            rel_path = os.path.relpath(posixpath.join(self.path, item), self._project.path)
-            if rel_path == '.':
-                return self._project
-            return self._project[rel_path]
-
-    def __getattr__(self, item):
-        if item in ['list_nodes', 'list_groups']:
-            try:
-                return getattr(self._wrapped_object, item)
-            except AttributeError:
-                return self._empty_list
-        return getattr(self._wrapped_object, item)
-
-    def __repr__(self):
-        return repr(self._wrapped_object)
-
-    def self_representation(self):
-        """Self representation of the wrapped object if known, else None is returned."""
-        if self._type is None:
-            return
-        if self._type == "structure":
-            return self._wrapped_object.plot3d()
-        if self._type == 'murnaghan':
-            plt.ioff()
-            self._wrapped_object.plot()
-
-
 class DisplayOutputGUI:
 
     """Display various kind of data in an appealing way using a ipywidgets.Output inside an ipywidgets.Vbox
     The behavior is very similar to standard ipywidgets.Output except one has to pass cls.box to get a display."""
     def __init__(self, *args, **kwargs):
         self.box = widgets.VBox(*args, **kwargs)
-        self.buttons = widgets.HBox()
         self.output = widgets.Output(layout=widgets.Layout(width='100%'))
-        self.fig = None
-        self.ax = None
+        self._display_obj = None
         self._debug = False
         self.refresh()
 
     def refresh(self):
-        self.box.children = (self.buttons, self.output)
+        self.box.children = [self.output]
 
     def __enter__(self):
         """Use context manager on the widgets.Output widget"""
-        self.buttons = widgets.HBox()
         return self.output.__enter__()
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -138,42 +56,33 @@ class DisplayOutputGUI:
         return self.output.__getattribute__(item)
 
     def clear_output(self, *args, **kwargs):
-        clear_button = kwargs.pop('clear_button', False)
-        if clear_button:
-            self.buttons = widgets.HBox()
         self.output.clear_output(*args, **kwargs)
         self.refresh()
 
-    def _update_buttons(self, obj):
-
-        def click_button(b):
-            self.output.clear_output()
-            self.display(obj=obj)
-
-        if isinstance(obj, PyironWrapper) and obj.has_self_representation:
-            button = widgets.Button(description="Re-plot " + obj.name)
-            button.on_click(click_button)
-            self.buttons.children = tuple([button])
-
-        self.refresh()
-
     def display(self, obj, default_output=None):
-        self._update_buttons(obj)
+        if isinstance(obj, BaseWrapper):
+            self.display(obj.gui, default_output=default_output)
+        elif isinstance(obj, np.ndarray):
+            self.display(NumpyWidget(obj), default_output=default_output)
+        else:
+            self._display_obj = obj
+            self._display(default_output)
+
+    def _display(self, default_output):
         with self.output:
-            if obj is None and default_output is None:
+            if self._display_obj is None and default_output is None:
                 raise TypeError("Given 'obj' is of 'NoneType'.")
-            elif obj is None:
+            elif self._display_obj is None:
                 print(default_output)
-            elif isinstance(obj, PyironWrapper):
-                plt.ioff()
-                to_display = obj.self_representation()
-                if to_display is not None:
-                    display(obj.self_representation())
+            elif isinstance(self._display_obj, ObjectWidget):
+                display(self._display_obj.gui)
             else:
                 plt.ioff()
-                display(self._output_conv(obj))
+                display(self._output_conv())
+
+    def _output_conv(self):
+        obj = self._display_obj
 
-    def _output_conv(self, obj):
         eol = os.linesep
         if self._debug:
             print('node: ', type(obj))
@@ -202,41 +111,17 @@ class DisplayOutputGUI:
                            eol + ' .... file too long: skipped ....')
         elif isinstance(obj, list):
             return pandas.DataFrame(obj, columns=['list'])
-        elif isinstance(obj, np.ndarray):
-            return self.plot_array(obj)
         elif str(type(obj)).split('.')[0] == "<class 'PIL":
             try:
                 data_cp = obj.copy()
                 data_cp.thumbnail((800, 800))
+                data_cp = data_cp.convert('RGB')
             except:
                 data_cp = obj
             return data_cp
         else:
             return obj
 
-    def plot_array(self, val):
-        if self.fig is None:
-            self.fig, self.ax = plt.subplots()
-        else:
-            self.ax.clear()
-
-        if val.ndim == 1:
-            self.ax.plot(val)
-        elif val.ndim == 2:
-            if len(val) == 1:
-                self.ax.plot(val[0])
-            else:
-                self.ax.plot(val)
-        elif val.ndim == 3:
-            self.ax.plot(val[:, :, 0])
-        else:
-            print(f"This is a numpy array of dim = {val.ndim}, " +
-                  "however, plotting is only implemented for up to 3-dimensions.\n")
-            return val
-
-        # self.ax.set_title(self._node_name)
-        return self.ax.figure
-
 
 class ProjectBrowser:
 
@@ -358,10 +243,13 @@ class ProjectBrowser:
     @property
     def _project_root_path(self):
         try:
-            root_path = self.project.root_path
+            return self.project.root_path
         except AttributeError:
-            root_path = self.project.project.root_path
-        return root_path
+            pass
+        try:
+            return self.project.project.root_path
+        except AttributeError:
+            return None
 
     def _busy_check(self, busy=True):
         """Function to disable widget interaction while another update is ongoing."""
@@ -393,7 +281,7 @@ class ProjectBrowser:
     def refresh(self):
         """Refresh the project browser."""
         self.output.clear_output(True)
-        if hasattr(self.project, 'self_representation'):
+        if isinstance(self.project, BaseWrapper):
             # print(f"has self_repr! and type of the wrapped obj = {type(self.project._wrapped_object)}")
             self.output.display(self.project)
 
@@ -442,13 +330,13 @@ class ProjectBrowser:
         set_path_button.on_click(click_option_button)
         if self.fix_path:
             set_path_button.disabled = True
-        childs = [set_path_button, self.path_string_box]
+        children = [set_path_button, self.path_string_box]
 
         button = widgets.Button(description="Reset selection", layout=widgets.Layout(width='min-content'))
         button.on_click(click_option_button)
-        childs.append(button)
+        children.append(button)
 
-        optionbox.children = tuple(childs)
+        optionbox.children = tuple(children)
 
     def _click_option_button(self, b):
         self.output.clear_output(True)
@@ -477,7 +365,7 @@ class ProjectBrowser:
     def data(self):
         if self._data is not None:
             return self._data
-        elif isinstance(self.project, PyironWrapper):
+        elif isinstance(self.project, BaseWrapper):
             return self.project._wrapped_object
         else:
             return None
@@ -506,11 +394,12 @@ class ProjectBrowser:
         if isinstance(path, str):
             rel_path = os.path.relpath(path, self.path)
             if rel_path == '.':
+                self.refresh()
                 return
             self._update_project_worker(rel_path)
         else:
             self._project = path
-        self.output.clear_output(True, clear_button=True)
+        self.output.clear_output(True)
         self.refresh()
 
     def _gen_pathbox_path_list(self):
